@@ -7,21 +7,12 @@ import multiprocessing
 import time
 import warnings
 from collections import defaultdict
-from typing import (
-    Any,
-    Coroutine,
-    DefaultDict,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Tuple,
-    Union,
-)
+from types import CoroutineType
+from typing import Any, DefaultDict, Dict, Iterable, List, Optional, Union
 
 import httpx
 import pandas as pd
-from pandas.core.groupby import DataFrameGroupBy
+from pydantic import version as pydantic_version
 from shorthand_datetime import parse_shorthand_datetime
 from tqdm import tqdm
 from tqdm.asyncio import tqdm_asyncio
@@ -32,10 +23,11 @@ from . import exceptions as E
 from . import version
 
 BASE_URL = "https://api.24sea.eu/routes/v1/"
-PYDANTIC_V2 = version.parse_version(version.__version__).major >= 2
+PYDANTIC_V2 = version.parse_version(pydantic_version.VERSION).major >= 2
 
 if PYDANTIC_V2:
-    from pydantic import BaseModel, field_validator, validate_call  # noqa: F401
+    from pydantic import field_validator  # type: ignore
+    from pydantic import BaseModel, validate_call
 
 else:
     from pydantic import BaseModel, validator  # noqa: F401
@@ -72,7 +64,7 @@ def handle_request(
     url: str,
     params: Dict,
     auth: Optional[httpx.BasicAuth],
-    headers: Dict,
+    headers: Optional[Dict[str, str]] = {"accept": "application/json"},
     max_retries: int = 10,
     timeout: int = 3600,
 ) -> httpx.Response:
@@ -115,41 +107,41 @@ def handle_request(
                 time.sleep(3)
                 continue
         except httpx.RequestError as exc:
+            # Bubble up network/timeout errors for caller to handle
             raise exc
-    # fmt: off
-    if r_.status_code in [400, 401, 403, 404, 503, 504]:
-        print(f"Request failed because: \033[31;1m{r_.text}\033[0m")
-        r_.raise_for_status()
-    elif r_.status_code in [500, 501, 502]:
-        print("\033[31;1mServer-side error. Try to run again the query. If the"
-              "error persists, you will need to contact support at "
-              "\033[32;1;4msupport.api@24sea.eu\033[0m")
-        r_.raise_for_status()
-    elif r_.status_code > 400:
-        print("Request failed with status code: "
-              f"\033[31;1m{r_.status_code}\033[0m")
-        r_.raise_for_status()
-    # fmt: on
+    # Validate status after finishing retry loop
+    E.raise_for_status(r_)
     return r_
 
 
 async def handle_request_async(
     url: str,
-    params: dict,
+    params: Dict,
     auth: Optional[httpx.BasicAuth],
-    headers: dict = {"accept": "application/json"},
+    headers: Optional[Dict[str, str]] = {"accept": "application/json"},
     max_retries: int = 10,
     timeout: int = 1800,
+    method: str = "GET",
+    json: Optional[Dict] = None,
 ) -> httpx.Response:
     """Asynchronously handle the request to the 24SEA API using httpx's
-    AsyncClient."""
+    AsyncClient. Supports GET, POST, PUT, DELETE methods."""
     retry_count = 0
     async with httpx.AsyncClient(
         auth=auth, headers=headers, timeout=timeout
     ) as client:
         while True:
             try:
-                r_ = await client.get(url, params=params)
+                if method.upper() == "GET":
+                    r_ = await client.get(url, params=params)
+                elif method.upper() == "POST":
+                    r_ = await client.post(url, params=params, json=json)
+                elif method.upper() == "PUT":
+                    r_ = await client.put(url, params=params, json=json)
+                elif method.upper() == "DELETE":
+                    r_ = await client.delete(url, params=params, json=json)
+                else:
+                    raise ValueError(f"Unsupported HTTP method: {method}")
                 if r_.status_code != 502 or retry_count >= max_retries:
                     break
                 retry_count += 1
@@ -158,20 +150,7 @@ async def handle_request_async(
                     continue
             except (httpx.NetworkError, httpx.TimeoutException) as exc:
                 raise exc
-        # fmt: off
-        if r_.status_code in [400, 401, 403, 404, 502, 503, 504]:
-            logging.error(f"Request failed because: \033[31;1m{r_.text}\033[0m")
-            r_.raise_for_status()
-        elif r_.status_code == 500:
-            logging.error("\033[31;1mInternal server error. You will need to "
-                          "contact support at \033[32;1;4msupport.api@24sea.eu"
-                          "\033[0m")
-            r_.raise_for_status()
-        elif r_.status_code > 400:
-            logging.error("Request failed with status code: "
-                          f"\033[31;1m{r_.status_code}\033[0m")
-            r_.raise_for_status()
-        # fmt: on
+        E.raise_for_status(r_)
         return r_
 
 
@@ -297,7 +276,7 @@ def parse_timestamp(
         # is not compatible with the formats provided, therefore try
         # with the following formats.
         formats = ["%Y-%m-%dT%H:%M:%S%z", "%d.%m.%YT%H:%M:%S.%f%z",
-                   "%Y-%m-%dT%H:%M:%SZ", "%d.%m.%YT%H:%M:%S.%fZ"]
+                   "%Y-%m-%dT%H:%M:%SZ", "%d.%m.%YT%H:%M:%S.%fZ", "%Y-%m"]
         # fmt: on
         df["timestamp"] = pd.NaT
         for fmt in formats:
@@ -317,6 +296,7 @@ def estimate_chunk_size(
     end_timestamp: Union[str, datetime.datetime],
     grouped_metrics: Iterable,
     selected_metrics: Union[pd.DataFrame, None] = None,
+    target: str = "metric",
 ):
     """
     Estimate the optimal chunk size for processing tasks based on the expected
@@ -339,6 +319,8 @@ def estimate_chunk_size(
     selected_metrics : pandas.DataFrame or None
         DataFrame containing selected metrics with at least a "metric" column
         and optionally a "data_group" column.
+    target : str, default "metric"
+        The target column name in `selected_metrics` and `grouped_metrics`
 
     Returns
     -------
@@ -361,20 +343,22 @@ def estimate_chunk_size(
         if isinstance(dt, str):
             try:
                 return pd.to_datetime(dt)
-            except pd._libs.tslibs.parsing.DateParseError:
-                dt = parse_shorthand_datetime(dt).replace(tzinfo=None)
+            except pd._libs.tslibs.parsing.DateParseError:  # type: ignore
+                _dt = parse_shorthand_datetime(dt)
+                if _dt is not None:
+                    dt = _dt.replace(tzinfo=None)
         return dt
 
     start_dt = parse_dt(start_timestamp)
     end_dt = parse_dt(end_timestamp)
-    n_minutes = (end_dt - start_dt).total_seconds() / 60
+    n_minutes = (end_dt - start_dt).total_seconds() / 60  # type: ignore
     n_points = int(n_minutes // 10) + 1
     n_tasks = len(tasks)
     # Build a dictionary of bytes per metric
     bytes_per_metric = {}
     if selected_metrics is not None:
         for _, row in selected_metrics.iterrows():
-            metric = row["metric"]
+            metric = row[target]
             data_group = str(row.get("data_group", "")).lower()
             if data_group == "fatigue":
                 bytes_per_metric[metric] = 200
@@ -387,9 +371,9 @@ def estimate_chunk_size(
     total_bytes = 0
     for _, group in grouped_metrics:
         if isinstance(group, pd.DataFrame):
-            group_met = group["metric"].tolist()
+            group_met = group[target].tolist()
         else:
-            group_met = [group["metric"]] if hasattr(group, "metric") else []
+            group_met = [group[target]] if hasattr(group, target) else []
         group_bytes = sum(bytes_per_metric.get(m, 8) for m in group_met)
         total_bytes += n_points * group_bytes
 
@@ -417,7 +401,7 @@ def estimate_chunk_size(
 
 
 async def gather_in_chunks(
-    tasks: List[Coroutine], chunk_size: int = 5, timeout: int = 3600
+    tasks: List[CoroutineType], chunk_size: int = 5, timeout: int = 3600
 ) -> List:
     results = []
     chunk_results = []
@@ -436,6 +420,7 @@ async def gather_in_chunks(
             range(0, len(tasks), max(1, chunk_size)),
             desc=desc,
             colour="#c9cfd8",
+            position=0,
         ):
             chunk = tasks[i : i + chunk_size]
             chunk_results = await tqdm_asyncio.gather(
@@ -443,6 +428,7 @@ async def gather_in_chunks(
                 desc=f"Getting chunk: [{i+1}-{i+len(chunk)}]",
                 timeout=timeout,
                 colour="#e4e8ee",
+                position=1,
             )
             results.extend(chunk_results)
     return results
@@ -454,16 +440,17 @@ def fetch_data_sync(
     location: str,
     start_timestamp: Union[datetime.datetime, str],
     end_timestamp: Union[datetime.datetime, str],
-    headers: Dict[str, str],
-    group: DataFrameGroupBy,
-    auth: httpx.BasicAuth,
+    headers: Optional[Dict[str, str]],
+    group: pd.DataFrame,
+    auth: Optional[httpx.BasicAuth],
     timeout: int,
+    target: str = "metric",
 ) -> Any:
     """Syncronously fetch metrics data for the datasignals API app."""
     # fmt: off
-    s_ = "• " + ",".join(group["metric"].tolist()).replace(",", "\n            • ")  # noqa: E501  # pylint: disable=C0301
+    s_ = "• " + ",".join(group[target].tolist()).replace(",", "\n            • ")  # noqa: E501  # pylint: disable=C0301
     logging.info(f"\033[32;1m⏳ Getting data for {site} - "
-                    f"{location}...\n📊 \033[35;1mMetrics: "
+                    f"{location}...\n📊 \033[35;1m{target.capitalize()}s: "
                     f"\033[0;34m{s_}\n\033[0m")
     # fmt: on
     r_ = handle_request(
@@ -473,7 +460,7 @@ def fetch_data_sync(
             "end_timestamp": end_timestamp,
             "project": [site],
             "location": [location],
-            "metrics": ",".join(group["metric"].tolist()),
+            f"{target}s": ",".join(group[target].tolist()),
         },
         auth,
         headers,
@@ -487,26 +474,114 @@ def fetch_data_sync(
     return r_.json()
 
 
+def fetch_availability_sync(
+    url,
+    site: str,
+    location: str,
+    start_timestamp: Union[datetime.datetime, str],
+    end_timestamp: Union[datetime.datetime, str],
+    granularity: Union[str, int],
+    sampling_interval_seconds: int,
+    headers: Optional[Dict[str, str]],
+    group: pd.DataFrame,
+    auth: Optional[httpx.BasicAuth],
+    timeout: int,
+) -> Any:
+    """Syncronously fetch metrics data for the datasignals API app."""
+    # fmt: off
+    s_ = "• " + ",".join(group["metric"].tolist()).replace(",", "\n            • ")  # noqa: E501  # pylint: disable=C0301
+    logging.info(f"\033[32;1m⏳ Getting availabilities for {site} - "
+                    f"{location}...\n📊 \033[35;1mMetrics: "
+                    f"\033[0;34m{s_}\n\033[0m")
+    params: Dict[str, Union[List[str], datetime.datetime, str, int]] = {
+        "start_timestamp": start_timestamp,
+        "end_timestamp": end_timestamp,
+        "project": [site],
+        "location": [location],
+        "metrics": ",".join(group["metric"].tolist()),
+    }
+    # fmt: on
+    if isinstance(granularity, str):
+        params["granularity"] = granularity
+    if isinstance(granularity, int):
+        params["bucket_seconds"] = granularity
+    params["sampling_interval_seconds"] = sampling_interval_seconds
+
+    r_ = handle_request(url, params, auth, headers, timeout=timeout)
+    # Warn if empty
+    if r_.json() == []:
+        logging.warning(
+            f"\033[33;1m⚠️ No data found for {site} - " f"{location}.\033[0m"
+        )
+    return r_.json()
+
+
+async def fetch_availability_async(
+    url: str,
+    site: str,
+    location: str,
+    start_timestamp: Union[datetime.datetime, str],
+    end_timestamp: Union[datetime.datetime, str],
+    granularity: Union[str, int],
+    sampling_interval_seconds: int,
+    headers: Optional[Dict[str, str]],
+    group: pd.DataFrame,
+    auth: Optional[httpx.BasicAuth],
+    timeout: int,
+    max_retries: int,
+) -> Any:
+    """Asynchronously fetch availability data for a site/location."""
+    s_ = "• " + ",".join(group["metric"].tolist()).replace(
+        ",", "\n            • "
+    )
+    logging.info(
+        f"\033[32;1m⏳ Getting availabilities for {site} - {location}..."
+        f"\n📊 \033[35;1mMetrics: \033[0;34m{s_}\n\033[0m"
+    )
+    params: Dict[str, Union[List[str], datetime.datetime, str, int]] = {
+        "start_timestamp": start_timestamp,
+        "end_timestamp": end_timestamp,
+        "project": [site],
+        "location": [location],
+        "metrics": ",".join(group["metric"].tolist()),
+    }
+    if isinstance(granularity, str):
+        params["granularity"] = granularity
+    else:
+        params["bucket_seconds"] = granularity
+    params["sampling_interval_seconds"] = sampling_interval_seconds
+    r_ = await handle_request_async(
+        url, params, auth, headers, max_retries=max_retries, timeout=timeout
+    )
+    result_json = r_.json()
+    if result_json == []:
+        logging.warning(
+            f"\033[33;1m⚠️ No data found for {site} - {location}.\033[0m"
+        )
+    return result_json
+
+
 async def fetch_data_async(
     url,
     site: str,
     location: str,
     start_timestamp: Union[datetime.datetime, str],
     end_timestamp: Union[datetime.datetime, str],
-    headers: Dict[str, str],
-    group: DataFrameGroupBy,
-    auth: httpx.BasicAuth,
+    headers: Optional[Dict[str, str]],
+    group: pd.DataFrame,
+    auth: Optional[httpx.BasicAuth],
     timeout: int,
     max_retries: int,
     as_dict: bool = False,
+    target: str = "metric",
 ) -> Union[pd.DataFrame, Dict[str, Any]]:
     """Asyncronously fetch metrics data for the datasignals API app."""
-    s_ = "• " + ",".join(group["metric"].tolist()).replace(
+    s_ = "• " + ",".join(group[target].tolist()).replace(
         ",", "\n            • "
     )
     logging.info(
         f"\033[32;1m⏳ Getting data for {site} - {location}..."
-        f"\n📊 \033[35;1mMetrics: \033[0;34m{s_}\n\033[0m"
+        f"\n📊 \033[35;1m{target.capitalize()}s: \033[0;34m{s_}\n\033[0m"
     )
     r_ = await handle_request_async(
         url,
@@ -515,7 +590,7 @@ async def fetch_data_async(
             "end_timestamp": end_timestamp,
             "project": [site],
             "location": [location],
-            "metrics": ",".join(group["metric"].tolist()),
+            f"{target}s": ",".join(group[target].tolist()),
         },
         auth,
         headers,
@@ -537,7 +612,7 @@ async def fetch_oldest_timestamp_async(
     site: str,
     locations: Optional[str],
     headers: Dict[str, str],
-    auth: httpx.BasicAuth,
+    auth: Optional[httpx.BasicAuth],
     timeout: int,
     max_retries: int,
     as_dict: bool = False,
@@ -636,6 +711,9 @@ def parse_stats_list(stats_list: List[Dict[str, Any]]) -> pd.DataFrame:
             stats_df[c] = None
     stats_df = stats_df[cols_order]
 
+    if isinstance(stats_df, pd.Series):
+        stats_df = stats_df.to_frame().T  # Convert Series to DataFrame
+
     return stats_df.reset_index(drop=True)
 
 
@@ -708,77 +786,207 @@ def get_stats_as_dict(
         lambda: defaultdict(pd.DataFrame)
     )
     # fmt: off
-    for (s_, l_), group in stats_df.groupby(['site', 'location']):
-        s_ = s_.lower()
-        l_ = l_.upper()
+    for (s_, l_), group in stats_df.groupby(['site', 'location']):  # type: ignore  # pylint: disable=C0301  # noqa: E501
+        s_ = s_.lower()  # type: ignore
+        l_ = l_.upper()  # type: ignore
         result_dict[s_][l_] = group.drop(['site', 'location'],
                                             axis=1).reset_index(drop=True)
     # fmt: on
     return default_to_regular_dict(result_dict)
 
 
-def get_site_locations(
-    sites: Union[str, List[str]],
-    locations: Optional[Union[str, List[str]]],
-    metrics_overview: pd.DataFrame,
-) -> Tuple[List[str], List[List[str]]]:
+def get_metrics_data_df_as_dict(
+    metrics_data_df: pd.DataFrame, selected_metrics: pd.DataFrame
+) -> Dict[str, Dict[str, pd.DataFrame]]:
     """
-    Provided a set of sites and locations, as well as the metrics overview
-    table, retrieve matching locations for each site.
+    Convert the metrics DataFrame to a dictionary format.
 
     Parameters
     ----------
-    sites : str or List[str]
-        Site(s) to filter.
-    locations : str or List[str], optional
-        Location(s) to filter. If None, all locations for the site(s) are used.
-    metrics_overview : pd.DataFrame
-        DataFrame containing metrics overview information.
+    metrics_data_df : pd.DataFrame
+        DataFrame containing metrics data.
 
     Returns
     -------
-    Tuple[List[str], List[List[str]]]
-        A tuple containing a list of sites and a list of lists of locations.
+    Dict[str, Dict[str, pd.DataFrame]]
+        Dictionary with site and location as keys and metrics data as values.
     """
-    if isinstance(locations, list):
-        if len(locations) > 1 and any(loc is None for loc in locations):
-            # fmt: off
-            raise E.DataSignalsError("You cannot assign multiple sites and "
-                                        "locations to a list containing None")
-        # fmt: on
-        if len(locations) == 1 and not locations[0]:
-            locations = None
-    if isinstance(sites, str):
-        sites = [sites]
-    if isinstance(locations, str):
-        locations = [locations] if locations else None
+    # We need to reset the dataframe index to get the timestamp column
+    # as a column and not as the index.
+    metrics_data_df = parse_timestamp(metrics_data_df, keep_index_only=False)
+    # fmt: off
+    __melted = pd.melt(metrics_data_df, id_vars=["timestamp"],
+                        var_name="metric", value_name="value")
+    # Case-insensitive merge on metric (keep original 'metric' columns, only drop 'metric_lower')
+    __melted["metric_lower"] = __melted["metric"].str.lower()
+    _selected_metrics_ci = selected_metrics.copy()
+    _selected_metrics_ci["metric_lower"] = _selected_metrics_ci["metric"].str.lower()
+    __merged = pd.merge(__melted, _selected_metrics_ci, on="metric_lower", how="left")
 
-    site_locations = []
-    for site in sites:
-        site = site.lower()
-        filtered_overview = metrics_overview[
-            metrics_overview["site"].str.lower() == site
-        ]
-        if locations:
-            locations_lower = [loc.lower() for loc in locations]
-            filtered_overview = filtered_overview[
-                filtered_overview["location"].str.lower().isin(locations_lower)
-            ]
-        locs = filtered_overview["location"].unique().tolist()
-        # print(f"Found locations for site {site}: {locs}")
-        if not locs:
-            raise E.DataSignalsError(
-                f"No matching locations (f{locations}) "
-                f"found for site: {site}"
-            )
-        if not locations:
-            site_locations.append(locs)
-        else:
-            site_locations.append(
-                [
-                    loc
-                    for loc in locs
-                    if loc.lower() in map(str.lower, locations)
-                ]
-            )
-    return sites, site_locations
+    # Resolve possible duplicate metric columns (metric_x from melted, metric_y from selected)
+    if "metric_x" in __merged.columns and "metric_y" in __merged.columns:
+        # Prefer the casing from the selected metrics dataframe when available
+        __merged["metric"] = __merged["metric_y"].fillna(__merged["metric_x"])
+        __merged.drop(columns=[c for c in ["metric_x", "metric_y",
+                                           "metric_lower"]
+                               if c in __merged.columns], inplace=True)
+    else:
+        # Just drop helper column
+        if "metric_lower" in __merged.columns:
+            __merged.drop(columns=["metric_lower"], inplace=True)
+    __srt = (__merged[["timestamp", "site", "location", "metric", "value"]]  # type: ignore  # pylint: disable=C0301  # noqa: E501
+             .sort_values(by=["timestamp", "location"]).reset_index(drop=True))
+    __df = (__srt.pivot_table(index=["site", "location", "timestamp"],
+                              columns="metric", values="value",
+                              aggfunc="first",).reset_index())
+    # fmt: on
+    if __df.empty:
+        raise E.DataSignalsError(
+            "\033[31mThe \033[1mas_dict \033[22mmethod can only be called "
+            "when the DataFrame is not empty."
+        )
+    if not all(c_ in __df.columns for c_ in ["site", "location", "timestamp"]):
+        raise E.DataSignalsError(
+            "\033[31mThe \033[1mas_dict \033[22mmethod can only be called "
+            "when the site, location, and timestamp columns are available."
+        )
+    if not isinstance(__df.index, pd.RangeIndex):
+        raise E.DataSignalsError(
+            "\033[31mThe \033[1mas_dict \033[22mmethod can only be called "
+            "when the index is a \033[1mRangeIndex.\n"
+            "\033[22mThe index is currently a \033[1m"
+            f"{type(__df.index).__name__}.\n"
+        )
+    # Group the DataFrame by site and location
+    groups = __df.groupby(["site", "location"])
+    __dict: Dict[str, Dict[str, pd.DataFrame]] = {}
+    for (s_, l_), group in groups:
+        s_ = s_.lower()
+        l_ = l_.upper()
+        if s_ not in __dict:
+            __dict[s_] = {}
+        # Manipulate the group to remove columns with all NaN values
+        # and set the index to the timestamp column.
+        _df: pd.DataFrame = group.drop(["site", "location"], axis=1).dropna(
+            axis=1, how="all"
+        )
+        _df = parse_timestamp(_df)
+        _df = _df.sort_index()  # sort by timestamp
+        if l_ not in __dict[s_]:
+            __dict[s_][l_] = _df
+
+    return default_to_regular_dict(__dict)
+
+
+def series_to_type(
+    series: pd.Series, dtype: Union[str, type]
+) -> Union[pd.Series, pd.Timestamp]:
+    """
+    Convert a pandas Series to a specified data type.
+
+    Parameters
+    ----------
+    series : pd.Series
+        The Series to convert.
+    dtype : Union[str, type]
+        The data type to convert the series to.
+
+    Returns
+    -------
+    pd.Series
+        The Series converted to the specified data type.
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> s = pd.Series([1, 2, 3])
+    >>> column_to_type(s, float)
+    0    1.0
+    1    2.0
+    2    3.0
+    dtype: float64
+    """
+    if dtype == "datetime":
+        return pd.to_datetime(series)
+    if dtype in ("float", float, "int", int):
+        return pd.to_numeric(series, errors="coerce").astype(dtype)  # type: ignore  # pylint: disable=C301  # noqa: E501
+    if dtype in ("str", "string", str):
+        return series.astype(str)
+    return series
+
+
+def column_to_type(
+    data: pd.DataFrame, column: str, dtype: Union[str, type]
+) -> pd.DataFrame:
+    """
+    Convert a column in a DataFrame to a specified data type.
+
+    Parameters
+    ----------
+    data : pd.DataFrame
+        The DataFrame containing the column to convert.
+    column : str
+        The column to convert.
+    dtype : Union[str, type]
+        The data type to convert the column to.
+
+    Returns
+    -------
+    pd.DataFrame
+        The DataFrame with the column converted to the specified data type.
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> df = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]})
+    >>> column_to_type(df, 'A', float)
+       A  B
+    0  1  4
+    1  2  5
+    2  3  6
+    """
+    data[column] = series_to_type(data[column], dtype)
+    return data
+
+
+def calendar_monthly_availability(df: pd.DataFrame) -> pd.DataFrame:
+    """Convert a daily availability dataframe to a calendar monthly availability
+    dataframe.
+    The columns of the input dataframe are assumed to be daily availability
+    values (between 0 and 1). The output dataframe will have the same columns,
+    but the two indices will be year and month. The values will be the mean of
+    the daily availability values in that month.
+
+    Parameters
+    ----------
+    df : pd.DataFrame
+        Input dataframe with daily availability values.
+
+    Returns
+    -------
+    pd.DataFrame
+        Output dataframe with calendar monthly availability values.
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> data = {'timestamp': pd.date_range(start='2023-01-01', periods=90,
+    ...                                    freq='D'),
+    ...         'availability': [0.9, 0.8, 0.95] * 30}
+    >>> df = pd.DataFrame(data).set_index('timestamp')
+    >>> calendar_monthly_availability(df)
+                availability
+    timestamp
+    2023-01          0.883333
+    2023-02          0.883333
+    2023-03          0.883333
+    """
+    df = parse_timestamp(df)
+    df_monthly = df.resample("M").mean()
+
+    import warnings
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=UserWarning)
+        df_monthly.index = df_monthly.index.to_period("M").to_timestamp()  # type: ignore  # pylint: disable=C0301  # noqa: E501
+    return df_monthly
